

from frhd import Encode as En #import Encode.py (for encoding base32)

class Track():
    """Create a freerider track instance."""

    def __init__(self):
        self.trackdata = '' #This will hold the track's mathematical parts
        self.tracklist = [[],[],[]] #3 lists: one for physics lines, one for scenery, and one for powerups

    def insLine(self,x1,y1,x2,y2,typeofline):
        if typeofline == 'p': #physics
            self.tracklist[0] += [[x1,y1,x2,y2]]
        if typeofline == 's': #scenery
            self.tracklist[1] += [[x1,y1,x2,y2]]

    def insStar(self,x,y):
        self.tracklist[2] += [['C',x,y]]

    def insCheck(self,x,y):
        self.tracklist[2] += [['T',x,y]]

    def insSlowMo(self,x,y):
        self.tracklist[2] += [['S',x,y]]

    def insBomb(self,x,y):
        self.tracklist[2] += [['O',x,y]]

    def insGravity(self,x,y,rot):
        assert rot in range(360) #gravity has rotation too
        self.tracklist[2] += [['G',x,y,rot]]

    def insBoost(self,x,y,rot):
        assert rot in range(360)
        self.tracklist[2] += [['B',x,y,rot]]

    def genCode(self):
        self.trackdatalist = [[],[],[]] #holds raw data to be joined into frhd text

        for pline in self.tracklist[0]: #physics
            self.trackdatalist[0] += En.encline(pline[0],pline[1],pline[2],pline[3])

        for sline in self.tracklist[1]: #scenery
            self.trackdatalist[1] += En.encline(sline[0],sline[1],sline[2],sline[3])

        for pup in self.tracklist[2]: #powerups
            if len(pup) == 3: #if powerup does not have the rotation attribute
                self.trackdatalist[2] += En.encpup(pup[1],pup[2],pup[0])
            if len(pup) == 4: #if powerup does have rotation attribute
                self.trackdatalist[2] += En.encpupr(pup[1],pup[2],pup[3],pup[0])

        self.finalData = '' #this will be put into frhd

        for typ in self.trackdatalist: #type of object
            for indiv in typ: #individual object
                self.finalData += indiv[0]
            self.finalData += '#'#add object end marker

        return self.finalData

if __name__ == '__main__':
    my_track = Track()
    my_track.insLine(6,6,6,6,'p')
    my_track.insLine(6,6,6,6,'s')
    my_track.insStar(6,6)
    my_track.insCheck(6,6)
    my_track.insSlowMo(6,6)
    my_track.insBomb(6,6)
    my_track.insGravity(6,6,6)
    my_track.insBoost(6,6,6)
    my_track.genCode()
    print(my_track.genCode())




import re #need it

#These regexes match power ups, and power ups with rotation.
PUP_RE  = re.compile('(?P<type>(C|T|S|O)) (?P<x>-?\S*) (?P<y>-?[^ ,#])')
PUPR_RE = re.compile('(?P<type>(G|B)) (?P<x>-?\S*) (?P<y>-?\S*) (?P<rot>-?[^ ,#])')

def decline(ls): #Decode line
    ls = ls.split(' ')
    ls = filter(None, ls)
    print(ls)
    for obj in range(0,len(ls)):
        ls[obj] = b32d(ls[obj])
    return ls

def decpup(pup): #Decode powerup
    return PUP_RE.match(pup).groupdict()

def decpupr(pupr): #Decode powerup with rotation
    return PUPR_RE.match(pupr).groupdict()

def b32d(n): #Base 32 decode
    return int(n, 32)

if __name__ == '__main__':
    print(decline('-18 1i 18 1i 4m 1i'))




import decode as De #Decode.py - if you haven't already
                                   #go take a look ;)

def loadTrack(track):
    track[0] = track[0].split('#')
    track[0] = track[0][0].split(',')
    print(track[0])
    for obj in range(0,len(track[0])):
        track[0][obj] = De.decline(track[0][obj])
    print(track)

if __name__ == "__main__":
    tdata = open('tests/test.ft').read()
    loadTrack(tdata)



from setuptools import setup, find_packages

versionFile = "VERSION"
setup(name="frhd-python",
    version=1.1,
    description="Library to edit FRHD tracks",
    long_description=open("README.rst").read(),
    url="https://github.com/gaetgu/frhd",
    download_url="https://github.com/gaetgu/frhd/archive/v_1.1.tar.gz",
    install_requires=[
          'decode',
      ],
    author="Gaetgu",
    author_email="gabriel.ethan.phantom@gmail.com",
    license="MIT License",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Code Generators",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3 :: Only"
    ],
    keywords="development freeriderhd freerider code track tracks",
    packages=find_packages(exclude=["images"]),
)




def encline(x1,y1,x2,y2):   #encode line.
    return [['%s %s %s %s,' % (b32e(x1),b32e(y1),b32e(x2),b32e(y2))]]

def encpup(x,y,pupcode):
    #encode powerup without rotation
    return [['%s %s %s,' % (pupcode,b32e(x),b32e(y))]]

def encpupr(x,y,rot,pupcode):
    #encode powerup with rotation
    return [['%s %s %s %s,' % (pupcode,b32e(x),b32e(y),b32e(rot))]]

def b32e(numbera):
    #encode number. I struggled to find the right alphabet that frhd used for their encoding
    """Encode number in freerider base32."""

    alphabet = '0123456789abcdefghijklmnopqrstuv' #DO NOT CHANGE
    number = abs(numbera)
    base32 = ''
    while number:
        number, i = divmod(number, 32)
        base32 = alphabet[i] + base32
    if numbera < 0:
        base32 = '-'+base32
    return base32 or alphabet[0]
